import socket
import time
import json
import os

BROADCAST_IP = '255.255.255.255'
PORT = 50000

current_dir = os.path.dirname(os.path.abspath(__file__))  
messages_path = os.path.join(current_dir, 'message.json')  



while True:
    try:
        with open(messages_path, 'r') as file:
            content = file.read().strip()
            if not content:
                continue  
            message = json.loads(content)
    except (FileNotFoundError, json.JSONDecodeError):
        continue  
    if message.get('message') is None:
        pass
    else:
        if time.time() > message.get('expireTime', 0):
            with open(messages_path, 'w') as file:
                file.write("{}")
        else:
            print(f"Sending message... {message.get('message')}")
            sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
            data = json.dumps(message).encode('utf-8')
            sock.sendto(data, (BROADCAST_IP, PORT))
    time.sleep(0.5)



import socket
import json

PORT = 50000
seen_messeages = set()

sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
sock.bind(('', PORT))


def decrypt(encrypted_message, key):
    if len(key) != 4:
        raise ValueError("Key must be 4 characters long")
    
    for char in key:
        if not (char.isupper() or char.isdigit()):
            raise ValueError("Key must contain only uppercase letters and digits")
    
    decrypted = ""
    key_index = 0
    
    for char in encrypted_message:
        if char.isalpha():
            current_key_char = key[key_index % 4]
            
            if current_key_char.isupper():
                key_shift = ord(current_key_char) - ord('A')
            else:  # digit
                key_shift = int(current_key_char)
            
            if char.islower():
                decrypted += chr((ord(char) - ord('a') - key_shift) % 26 + ord('a'))
            else:
                decrypted += chr((ord(char) - ord('A') - key_shift) % 26 + ord('A'))
            
            key_index += 1
        else:
            decrypted += char
    
    return decrypted


def get_message_from_sender(code):
    decrypted_message = ""
    while True:
        data, addr = sock.recvfrom(1024)
        try:
            msg = json.loads(data.decode('utf-8'))
            message = msg.get('message', '')


            if  message not in seen_messeages:
                if message[0] == code[0]:
                    decrypted_message = decrypt(message[1:], code)
                    break
                else:
                    pass
                seen_messeages.add(message)


        except Exception as e:
            print(f"Error decoding message: {e}")
    return decrypted_message



import os
import click
import string
import random
import time
import json
import subprocess
import itertools
from . import cilent


current_dir = os.path.dirname(os.path.abspath(__file__))  
messages_path = os.path.join(current_dir, 'message.json')  

def encrypt(message, key):
    if len(key) != 4:
        raise ValueError("Key must be 4 characters long")
    
    for char in key:
        if not (char.isupper() or char.isdigit()):
            raise ValueError("Key must contain only uppercase letters and digits")
    
    encrypted = ""
    key_index = 0
    
    for char in message:
        if char.isalpha():
            current_key_char = key[key_index % 4]
            
            if current_key_char.isupper():
                key_shift = ord(current_key_char) - ord('A')
            else:
                key_shift = int(current_key_char)
            
            if char.islower():
                encrypted += chr((ord(char) - ord('a') + key_shift) % 26 + ord('a'))
            else:
                encrypted += chr((ord(char) - ord('A') + key_shift) % 26 + ord('A'))
            
            key_index += 1
        else:
            encrypted += char
    
    return encrypted

def generate_code(message):
    characters = string.ascii_uppercase + string.digits
    return ''.join(random.choice(characters) for _ in range(4))


@click.group()
def cli():
    pass

@click.command()
def init():
    current_dir = os.path.dirname(os.path.abspath(__file__))  
    messages_path = os.path.join(current_dir, 'message.json')  
    if not os.path.exists(messages_path):
        with open(messages_path, 'w') as file:
            file.write("{}")
    sender_path = os.path.join(current_dir, 'sender.py')
    subprocess.Popen(['cmd', '/c', f'start pythonw {sender_path}'])

@click.command()
@click.argument('message')
def send(message):
    if not os.path.exists(messages_path):
        print("Please initialize the LocalDrop Using 'localdrop init' first.")
    else:
        code = generate_code(message)
        c = code
        message_dict = {
            "expireTime":time.time() + 60,
            "message": code[0]+encrypt(message, c),
        }
        print(f"Your Code: {code}")
        with open(messages_path, 'w') as file:
            json.dump(message_dict, file, ensure_ascii=False, indent=4)

@click.command()
@click.argument('code')
def get(code):
    message = cilent.get_message_from_sender(code)
    if message:
        print(f"{message}")
    else:
        print("No message found for the given code.")

cli.add_command(init)
cli.add_command(send)
cli.add_command(get)

if __name__ == "__main__":
    cli()